# MaxN.py     I. Marti-Vidal.      IMPRS Blackboard Lectures 2011.
# COMPUTE THE DISTRIBUTION OF THE MAXIMUM EXPECTED VALUE OF A SAMPLE OF RANDOM NORMAL VARIABLES.
# (THIS IS USEFUL TO ESTIMATE THE STATISTICAL SIGNIFICANCE OF A SOURCE IN A NOISY IMAGE).

from scipy.stats import norm
import numpy
import pylab

# Parameters:
######################################################################
npix =  512            # Size of the plot of probability density (pixels)
sigs = [1.0, 5.0]      # Range of sigmas (i.e., noise levels) to compute (min, max).
nums = [1.e+1, 1.e+4]  # Range of number of samples from which to take the maximum.
######################################################################


# PROGRAM STARTS:

# Set the simulation space (log in sigma, linear in number of samples):

nnums = numpy.array([nums[0]*numpy.exp(float(i)/(npix-1)*numpy.log(nums[1])) for i in range(npix)])
nsigs = numpy.linspace(sigs[0],sigs[1],npix)
delts = (sigs[1]-sigs[0])/npix

# Compute pdf (and cdf) of the maximum values of the samples (see details in the lecture):

results = []
ccdf = norm.cdf(nsigs)
ppdf = norm.pdf(nsigs)
for nd in nnums:
 res1 = numpy.power(ccdf,nd)*ppdf
# Normalize CDF (i.e., definite integral = 1):
 maxres = res1/(sum(res1)*delts)
 results.append(maxres)


# Prepare axes for plot:
Xax = numpy.outer(numpy.ones(npix),nsigs)
Yax = numpy.outer(nnums,numpy.ones(npix))

# Plot PDF:
plot1 = pylab.pcolormesh(Xax,Yax,numpy.array(results),vmin=0,vmax=numpy.max(results))
pylab.colorbar()
pylab.title("Probability density of peak (pure Gaussian noise)")
pylab.xlabel("Dynamic range (i.e., peak / sigma )")
pylab.ylabel("# of sample points")
pylab.axis([nsigs[0],nsigs[-1],nnums[0],nnums[-1]])

# Plot complementary of CDF:
pylab.figure()
pylab.pcolormesh(Xax,Yax,numpy.array([1.-delts*numpy.cumsum(results[i]) for i in range(npix)]),vmin=0,vmax=1.0)
pylab.title("Complementary cumulative function (pure Gaussian noise)")
pylab.xlabel("Dynamic range (i.e., peak / sigma )")
pylab.ylabel("# of sample points")
pylab.colorbar()
pylab.axis([nsigs[0],nsigs[-1],nnums[0],nnums[-1]])

pylab.show()



